<% if @use_amavisd %>
amavis unix - - <%= @jail %> - <%= @amavis_procs %> smtp
        -o smtp_data_done_timeout=1200
        -o smtp_send_xforward_command=yes

127.0.0.1:10025 inet n - <%= @jail %> - - smtpd
        -o content_filter=
        -o local_recipient_maps=
        -o relay_recipient_maps=
        -o smtpd_restriction_classes=
        -o smtpd_client_restrictions=
        -o smtpd_helo_restrictions=
        -o smtpd_sender_restrictions=
        -o smtpd_recipient_restrictions=permit_mynetworks,reject
        -o mynetworks=127.0.0.0/8
        -o strict_rfc821_envelopes=yes
        -o receive_override_options=no_unknown_recipient_checks,no_header_body_checks
        -o smtp_bind_address=127.0.0.1
<% end -%>
<% if @use_dovecot_lda %>
dovecot   unix  -       n       n       -       -       pipe
  flags=DRhu user=<%= @mail_user %>:<%= @mail_user %> argv=/usr/lib/dovecot/deliver -d ${recipient}
<% end -%>
<% if @use_schleuder %>
<% if @use_schleuder == 2 %>
schleuder  unix  -       n       n       -       -       pipe
  flags=DRhu user=schleuder argv=/usr/bin/schleuder ${user}
<% else %>
schleuder  unix  -       n       n       -       -       pipe
  flags=DRhu user=schleuder argv=/usr/bin/schleuder work ${recipient}
<% end -%>
<% end -%>
<% if @use_sympa %>
sympa        unix  -       n       n       -       -       pipe
  flags=R user=sympa argv=/usr/lib/sympa/bin/queue ${recipient}
sympabounce  unix  -       n       n       -       -       pipe
  flags=R user=sympa argv=/usr/lib/sympa/bin/bouncequeue ${user}
<% end -%>
<%= @master_entries.join("\n") %>
