# file managed by puppet
#
# Postfix master process configuration file.  For details on the format
# of the file, see the master(5) manual page (command: "man 5 master").
#
# ==========================================================================
# service type  private unpriv  chroot  wakeup  maxproc command + args
#               (yes)   (yes)   (yes)   (never) (100)
# ==========================================================================
<% if @master_smtp -%>
<%= @master_smtp %>
<% elsif @smtp_listen == 'all' -%>
smtp      inet  n       -       <%= @jail %>       -       -       smtpd
<% else -%>
  <%- (@smtp_listen.is_a?(Array) ? @smtp_listen : @smtp_listen.split(' ')).each do |listen_address| -%>
<%= listen_address %>:smtp      inet  n       -       <%= @jail %>       -       -       smtpd
  <%- end -%>
<% end -%>
<% if @master_submission -%>
<%= @master_submission %>
<% end -%>
<% if @master_smtps -%>
<%= @master_smtps %>
<% end -%>
#smtp      inet  n       -       n       -       -       smtpd
#submission inet n       -       n       -       -       smtpd
#  -o smtpd_enforce_tls=yes
#  -o smtpd_sasl_auth_enable=yes
#  -o smtpd_client_restrictions=permit_sasl_authenticated,reject
#smtps     inet  n       -       n       -       -       smtpd
#  -o smtpd_tls_wrappermode=yes
#  -o smtpd_sasl_auth_enable=yes
#  -o smtpd_client_restrictions=permit_sasl_authenticated,reject
#628      inet  n       -       n       -       -       qmqpd
pickup    fifo  n       -       <%= @jail %>       60      1       pickup
cleanup   unix  n       -       <%= @jail %>       -       0       cleanup
qmgr      fifo  n       -       n       300     1       qmgr
#qmgr     fifo  n       -       n       300     1       oqmgr
tlsmgr    unix  -       -       <%= @jail %>       1000?   1       tlsmgr
rewrite   unix  -       -       <%= @jail %>       -       -       trivial-rewrite
bounce    unix  -       -       <%= @jail %>       -       0       bounce
defer     unix  -       -       <%= @jail %>       -       0       bounce
trace     unix  -       -       <%= @jail %>       -       0       bounce
verify    unix  -       -       <%= @jail %>       -       1       verify
flush     unix  n       -       <%= @jail %>       1000?   0       flush
proxymap  unix  -       -       n       -       -       proxymap
smtp      unix  -       -       <%= @jail %>       -       -       smtp
# When relaying mail as backup MX, disable fallback_relay to avoid MX loops
relay     unix  -       -       <%= @jail %>       -       -       smtp
	-o fallback_relay=
#       -o smtp_helo_timeout=5 -o smtp_connect_timeout=5
showq     unix  n       -       <%= @jail %>       -       -       showq
error     unix  -       -       <%= @jail %>       -       -       error
discard   unix  -       -       <%= @jail %>       -       -       discard
local     unix  -       n       n       -       -       local
virtual   unix  -       n       n       -       -       virtual
lmtp      unix  -       -       <%= @jail %>       -       -       lmtp
anvil     unix  -       -       <%= @jail %>       -       1       anvil
scache	  unix	-	-	<%= @jail %>	-	1	scache
#
# ====================================================================
# Interfaces to non-Postfix software. Be sure to examine the manual
# pages of the non-Postfix software to find out what options it wants.
#
# Many of the following services use the Postfix pipe(8) delivery
# agent.  See the pipe(8) man page for information about ${recipient}
# and other message envelope options.
# ====================================================================
#
# maildrop. See the Postfix MAILDROP_README file for details.
# Also specify in main.cf: maildrop_destination_recipient_limit=1
#
maildrop  unix  -       n       n       -       -       pipe
  flags=DRhu user=<%= @mail_user %> argv=/usr/local/bin/maildrop -d ${recipient}
#
# The Cyrus deliver program has changed incompatibly, multiple times.
#
old-cyrus unix  -       n       n       -       -       pipe
  flags=R user=cyrus argv=/usr/lib/cyrus-imapd/deliver -e -m ${extension} ${user}
# Cyrus 2.1.5 (Amos Gouaux)
# Also specify in main.cf: cyrus_destination_recipient_limit=1
cyrus     unix  -       n       n       -       -       pipe
  user=cyrus argv=/usr/lib/cyrus-imapd/deliver -e -r ${sender} -m ${extension} ${user}
#
# See the Postfix UUCP_README file for configuration details.
#
uucp      unix  -       n       n       -       -       pipe
  flags=Fqhu user=uucp argv=uux -r -n -z -a$sender - $nexthop!rmail ($recipient)
#
# Other external delivery methods.
#
ifmail    unix  -       n       n       -       -       pipe
  flags=F user=ftn argv=/usr/lib/ifmail/ifmail -r $nexthop ($recipient)
bsmtp     unix  -       n       n       -       -       pipe
  flags=Fq. user=foo argv=/usr/local/sbin/bsmtp -f $sender $nexthop $recipient
